/* 
 * Tenshi no Hashigo for X
 * Copyright (c) 2001 TF <tf@denpa.org>
 * All rights reserverd.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *      This product includes software developed by TF.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: $
 */
#include <X11/Xlib.h>

#include "../aladder.h"

#define XOFFSET 24
#define YOFFSET 16

#define XMAX 27
#define YMAX 16
#define XPAD 0
#define YPAD 3

typedef struct {
    Display *disp;
    GC gc;
    XFontStruct *xfont;
    int width;
    int top;
    int pitch;
} FontInfo;

static FontInfo finfo;

static long black, white, gray, yellow;

static long
GetPixelByName(Display *disp, const char *name)
{
    Colormap cmap;
    XColor c0, c1;

    cmap = DefaultColormap(disp, 0);
    XAllocNamedColor(disp, cmap, name, &c1, &c0);

    return c1.pixel;
}

void
InitFont(Display *disp, GC gc, XFontStruct *f)
{
    if (f != NULL && f->fid != None) {
	XSetFont(disp, gc, f->fid);
    } else {
	fprintf(stderr, "Can't get the font.\n");
	exit(1);
    }
    finfo.disp = disp;
    finfo.gc = gc;
    finfo.xfont = f;
    finfo.width = f->max_bounds.rbearing + XPAD;
    finfo.top = f->max_bounds.ascent;
    finfo.pitch = f->max_bounds.ascent + f->max_bounds.descent + YPAD;

    black = GetPixelByName(disp, "black");
    white = GetPixelByName(disp, "white");
    gray = GetPixelByName(disp, "gray");
    yellow = GetPixelByName(disp, "yellow");
}

void
PutChar(Drawable dst, int code, int x, int y, int attr)
{
    XChar2b ch;
    long foreground;

    ch.byte1 = (code >> 8) & 0x7f;
    ch.byte2 = code & 0x7f;

    x = x * finfo.width + XOFFSET;
    y = y * finfo.pitch + finfo.top + YOFFSET;

    XSetForeground(finfo.disp, finfo.gc, black);
    switch (attr) {
    case ATTR_NORMAL:
	XDrawString16(finfo.disp, dst, finfo.gc, x - 1, y, &ch, 1);
	XDrawString16(finfo.disp, dst, finfo.gc, x, y - 1, &ch, 1);

	XDrawString16(finfo.disp, dst, finfo.gc, x + 1, y, &ch, 1);
	XDrawString16(finfo.disp, dst, finfo.gc, x, y + 1, &ch, 1);

	XDrawString16(finfo.disp, dst, finfo.gc, x + 1, y + 1, &ch, 1);
	XDrawString16(finfo.disp, dst, finfo.gc, x + 2, y + 2, &ch, 1);
	break;
    case ATTR_LINK:
    case ATTR_ALTN:
    case ATTR_LINK_HL:
    case ATTR_ALTN_HL:
	XDrawString16(finfo.disp, dst, finfo.gc, x - 1, y, &ch, 1);
	XDrawString16(finfo.disp, dst, finfo.gc, x, y - 1, &ch, 1);

	XDrawString16(finfo.disp, dst, finfo.gc, x + 1, y, &ch, 1);
	XDrawString16(finfo.disp, dst, finfo.gc, x, y + 1, &ch, 1);
	break;
    }

    switch (attr) {
    case ATTR_NORMAL:
    case ATTR_ALTN:
	foreground = white;
	break;
    case ATTR_LINK:
	foreground = gray;
	break;
    case ATTR_LINK_HL:
    case ATTR_ALTN_HL:
	foreground = yellow;
	break;
    }

    XSetForeground(finfo.disp, finfo.gc, foreground);
    XDrawString16(finfo.disp, dst, finfo.gc, x, y, &ch, 1);
}

void
text_coordinate(int x, int y, int *tx, int *ty)
{
    *tx = (x - XOFFSET) / finfo.width;
    *ty = (y - YOFFSET) / finfo.pitch;
    // fprintf(stderr, "[%d, %d, %d, %d]", y, YOFFSET, finfo.top,
    // finfo.pitch);

    if (x < XOFFSET || y < YOFFSET || *tx > TEXT_WIDTH
	|| *ty > TEXT_HEIGHT) {
	*tx = -1;
	*ty = -1;
    }
}
