/* 
 * Tenshi no Hashigo for X
 * Copyright (c) 2001 TF <tf@denpa.org>
 * All rights reserverd.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *      This product includes software developed by TF.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: $
 */

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Intrinsic.h>
#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>
#include <X11/Xlocale.h>
#include <X11/Shell.h>

#include <signal.h>

#include "../aladder.h"

#include "XLCore.h"

#define APPLICATIONCLASS "XLadder"

typedef struct {
    Bool screen_mode;
} OptionValue;

static XrmOptionDescRec options[] = {
#ifdef XVIDMODE
    {"-screen", "*screenMode", XrmoptionNoArg, (XtPointer) "True"},
#endif
};

static String fallbacks[] = {
    "*alcore.width: 640",
    "*alcore.height: 480",
    "*alcore.font: kanji24",
    "*name.international: False",
    NULL,
};

static OptionValue option_value;

static XtResource resources[] = {
#ifdef XVIDMODE
    {"screenMode", "ScreenMode", XtRBool, sizeof(Bool),
     offsetof(OptionValue, screen_mode), XtRImmediate, (XtPointer) False}
    ,
#endif
};

void quit_proc(Widget w, XEvent *e, String *p, Cardinal *n);

static XtActionsRec actions[] = {
    {"quit", (XtActionProc) quit_proc},
#if 0
    {"menupopup", (XtActionProc) MenuPopup},
#endif
#ifdef XVIDMODE
    {"scrwin", ScreenWindow}
#endif
};

String translations = "<Message>WM_PROTOCOLS: quit()";

String translations_core =
#ifdef XVIDMODE
    "Meta<Key>Return: scrwin()\n"
#endif
#if 0
    "<Btn3Down>: menupopup()"
#endif
    "Alt<Key>q: quit()\n";

Atom wm_protocols[2];

static void
signal_handler(int sig)
{
    XtAppSetExitFlag(alCore.x11_data.app);
}

void *
setup_signal(int signo, void *func(int))
{
    struct sigaction act, oact;
    act.sa_handler = func;
    sigemptyset(&act.sa_mask);
    act.sa_flags = 0;
    if (sigaction(signo, &act, &oact) < 0)
	return SIG_ERR;
    return oact.sa_handler;
}

void
quit_proc(Widget w, XEvent *e, String *p, Cardinal *n)
{
    if (e->xclient.data.l[0] == wm_protocols[0] ||
	e->xclient.data.l[0] == wm_protocols[1]) {
	XtAppSetExitFlag(XtWidgetToApplicationContext(w));
    }
}

void
x11_init(int argc, char *argv[])
{
    XtSetLanguageProc(NULL, NULL, NULL);
    alCore.x11_data.top =
	XtVaOpenApplication(&alCore.x11_data.app, APPLICATIONCLASS,
			    options, XtNumber(options), &argc, argv,
			    fallbacks, sessionShellWidgetClass, NULL);

    setup_signal(SIGINT, signal_handler);
    setup_signal(SIGTERM, signal_handler);

    XtVaGetApplicationResources(alCore.x11_data.top,
				(XtPointer) & option_value, resources,
				XtNumber(resources), NULL);
    XtAppAddActions(alCore.x11_data.app, actions, XtNumber(actions));
    XtOverrideTranslations((Widget) alCore.x11_data.top,
			   XtParseTranslationTable(translations));

    alCore.x11_data.core =
	XtVaCreateManagedWidget("alcore", xlCoreWidgetClass,
				alCore.x11_data.top, NULL);
    XtOverrideTranslations((Widget) alCore.x11_data.core,
			   XtParseTranslationTable(translations_core));

#ifdef XVIDMODE
    SetupVidMode(alCore.x11_data.top, option_value.screen_mode);
#endif

    // XtSetMappedWhenManaged(alCore.x11_data.top, False);
    XtRealizeWidget(alCore.x11_data.top);

    wm_protocols[0] =
	XInternAtom(XtDisplay(alCore.x11_data.top), "WM_DELETE_WINDOW",
		    False);
    wm_protocols[1] =
	XInternAtom(XtDisplay(alCore.x11_data.top), "WM_SAVEYOURSELF",
		    False);
    XSetWMProtocols(XtDisplay(alCore.x11_data.top),
		    XtWindow(alCore.x11_data.top), wm_protocols, 2);

    XtMapWidget(alCore.x11_data.top);
    // CreateBrowsers(alCore.x11_data.top);
#if XVIDMODE
    if (vm.enable_screen_mode && option_value.screen_mode) {
	XGrabPointer(XtDisplay(alCore.x11_data.top),
		     XtWindow(alCore.x11_data.top), True, 0, GrabModeAsync,
		     GrabModeAsync, XtWindow(alCore.x11_data.top), None,
		     CurrentTime);
    }
#endif
    RestrictWindowSize(alCore.x11_data.top);
}

void
x11_delete()
{
    XtUnrealizeWidget(alCore.x11_data.top);
    XtDestroyWidget(alCore.x11_data.top);
}

static void
RestrictWindowSize(Widget w)
{
    XSizeHints sh;

    sh.flags = PMinSize | PMaxSize;
    sh.min_width = WIDTH;
    sh.min_height = HEIGHT;
    sh.max_width = WIDTH;
    sh.max_height = HEIGHT;
    XSetWMNormalHints(XtDisplay(w), XtWindow(w), &sh);
}
