/* 
 * Tenshi no Hashigo for X
 * Copyright (c) 2001 TF <tf@denpa.org>
 * All rights reserverd.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *      This product includes software developed by TF.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: $
 */
#include <string.h>

#include "aladder.h"
#include "command.h"

static void
cmd_test(char arg[][ARG_LEN], int num)
{
    int i;

    fprintf(stderr, "[%s]", arg[0]);
    for (i = 1; i < num; i++) {
	fprintf(stderr, "[%s]", arg[i]);
    }
    fprintf(stderr, "\n");
}

static int
eval(char arg[][ARG_LEN], int num)
{
    int i;

    for (i = 0; i < num; i++) {
	int result = False;
	FlagList *p = alCore.flag;
	while (p != NULL) {
	    if (!strcmp(p->value, arg[i])) {
		result = True;
		break;
	    }
	    p = p->next;
	}
	if (!result) {
	    dprintf((stderr, "cond:<%s>=False\n", arg[i]));
	    return False;
	}
	dprintf((stderr, "cond:<%s>=True\n", arg[i]));
    }
    return True;
}

static u_char command_buf[SCR_BUF_LEN];

void
set_command(const char *cmd)
{
    strcpy(command_buf, cmd);
}

int
cmd_Bg(char arg[][ARG_LEN], int num)
{
  static int state = -1;
  static int lstate, dstate, abstate;

  if (state < 0) {
    state = lstate = dstate = abstate = 0;
    cmd_test(arg, num);
    ImageLoad(alCore.sub, arg[1]);
    strcpy(alCore.scene.bg, arg[1]);
  }

  if (lstate < 10) {
    Lighten(&lstate);
    return 0;
  }

  if (abstate < 20) {
    EffectAlphaBlend(&abstate);
    return 0;
  }

  if (dstate < 10) {
    Darken(&dstate);
    return 0;
  }
  
  if (!alCore.text.clear) redraw_text();
  state = -1;
  return 1;
}

/* 
 * CG λȺѤߥե饰ꤹ
 */
int
cmd_CGLook(char arg[][ARG_LEN], int num)
{
    cmd_test(arg, num);

    add_flag(arg[1], &alCore.cg_flag);
    return 1;
}

/*
 * 饯Ωɽ
 */
int
cmd_Chara(char arg[][ARG_LEN], int num)
{
  Image *img, *mask;
  char mask_name[ARG_LEN];

  cmd_test(arg, num);
    
  img = ImageCreate(0, 0);
  mask = ImageCreate(0, 0);
    
  sprintf(mask_name, "m%s", arg[1]);

  ImageLoad(img, arg[1]);
  ImageLoad(mask, mask_name);

  {
    int xoff;

    if (!strcmp(arg[2], "R")) {
      xoff = 0;
      strcpy(alCore.scene.right, arg[1]);
    } else if (!strcmp(arg[2], "M")) {
      xoff = WIDTH / 2 - img->width / 2;
      strcpy(alCore.scene.middle, arg[1]);
    } else if (!strcmp(arg[2], "L")) {
      xoff = WIDTH - img->width;
      strcpy(alCore.scene.left, arg[1]);
    } else {
      fprintf(stderr, "Ωɽֻ꤬Ǥ[%s]\n", arg[2]);
      exit(1);
    }
    ImageAddWithMask(alCore.work, img, mask, xoff, 0);
  }

    ImageDisplay(alCore.brightness);
    FlushWindow();
    redraw_text();

    ImageDestroy(img);
    ImageDestroy(mask);

    return 1;
}

int
cmd_Effect(char arg[][ARG_LEN], int num)
{
    cmd_test(arg, num);
    return 1;
}

int
cmd_End(char arg[][ARG_LEN], int num)
{
    cmd_test(arg, num);

    save_persistent_info();
    exit(1);
}

int
cmd_FadeOutMusic(char arg[][ARG_LEN], int num)
{
    cmd_test(arg, num);
    return 1;
}

/* 
 * ʥꥪʬѤΥե饰ΩƤ
 */
int
cmd_Flag(char arg[][ARG_LEN], int num)
{
    int i;
    cmd_test(arg, num);

    for (i = 1; i < num; i++) {
	add_flag(arg[i], &alCore.flag);
    }
    return 1;
}

int
cmd_IfJump(char arg[][ARG_LEN], int num)
{
    cmd_test(arg, num);

    if (num < 2) {
	fprintf(stderr, "郎ޤ\n");
	exit(1);
    }

    if (eval(arg + 2, num - 2)) {
	fprintf(stderr, "[<%s> إ]\n", arg[1]);
	cmd_Jump(arg, 1);
    }
    return 1;
}

int
cmd_Jump(char arg[][ARG_LEN], int num)
{
    SceneList *p;

    cmd_test(arg, num);

    p = alCore.script.scene_list;
    while (p != NULL) {
	if (!strcmp(p->name, arg[1])) {
	    alCore.script.cur_pos = p->offset;
	    fprintf(stderr, "٥[%s]: եå %d إ\n",
		    arg[1], p->offset);
	    return 1;
	}
	p = p->next;
    }
    fprintf(stderr, "٥[%s]ޤ\n", arg[1]);
    exit(1);
}

int
cmd_Light(char arg[][ARG_LEN], int num)
{
    cmd_test(arg, num);
    return 1;
}

int
cmd_MessageType(char arg[][ARG_LEN], int num)
{
    int type;
    cmd_test(arg, num);
    
    sscanf(arg[1], "%d", &type);
    if (type == 0) {
      fprintf(stderr, "[]\n");
      alCore.text.fullscreen = True;
      alCore.text.clear = True;
    } else if (type == 1) {
      fprintf(stderr, "[̲]\n");
      alCore.text.fullscreen = False;
      alCore.text.clear = True;
    } else {
      fprintf(stderr, "ʥåɽǤ\n");
      exit(1);
    }
    return 1;
}

int
cmd_Music(char arg[][ARG_LEN], int num)
{
    cmd_test(arg, num);
    return 1;
}

int
cmd_OutAllChara(char arg[][ARG_LEN], int num)
{
    cmd_test(arg, num);

    ImageLoad(alCore.work, alCore.scene.bg);
    ImageDisplay(alCore.brightness);
    //FlushWindow();
    redraw_text();

    return 1;
}

int
cmd_OutBg(char arg[][ARG_LEN], int num)
{
  int r, g, b;
  static int state = -1;
  static int lstate, dstate, abstate;

  if (state < 0) {
    state = lstate = dstate = abstate = 0;
    cmd_test(arg, num);
    sscanf(arg[1], "0x%02x%02x%02x", &r, &g, &b);

    ImageFill(alCore.sub, 0, 0, WIDTH, HEIGHT, r, g, b);
  }

  if (lstate < 10) {
    Lighten(&lstate);
    return 0;
  }

  if (abstate < 20) {
    EffectAlphaBlend(&abstate);
    return 0;
  }

  if (dstate < 10) {
    Darken(&dstate);
    return 0;
  }
  
  if (!alCore.text.clear) redraw_text();
  state = -1;
  return 1;
}

int
cmd_OutChara(char arg[][ARG_LEN], int num)
{
    cmd_test(arg, num);
    return 1;
}

int
cmd_PREVLook(char arg[][ARG_LEN], int num)
{
    cmd_test(arg, num);
    return 1;
}

/* 
 * ڡ
 */
int
cmd_Page(char arg[][ARG_LEN], int num)
{
    dprintf((stderr, "[ڡ]\n"));
    newpage();
    pop_stack();
    push_stack(STAT_WAIT);

    return 0;
}

int
cmd_Return(char arg[][ARG_LEN], int num)
{
    cmd_test(arg, num);
    return 1;
}

int
cmd_SceneName(char arg[][ARG_LEN], int num)
{
  // newpage();
  
  dprintf((stderr, "[: %s]\n", arg[1]));

    return 1;
}

int
cmd_SelStart(char arg[][ARG_LEN], int num)
{
    int i, j;
    cmd_test(arg, num);

    for (i = 0; i < alCore.alternative_num; i++) {
	for (j = 0; alCore.alternative[i].string[j] != 0 ; j++) {
	  put_char(alCore.alternative[i].string[j], i | ATTR_ALTN_MASK);
	  //debug_put_char(alCore.alternative[i].string[j]);
	}
	newline();
    }

    pop_stack();
    push_stack(STAT_ALTN);
    push_stack(STAT_EMPTY);

    return 1;
}

int
cmd_Select(u_char arg[][ARG_LEN], int num)
{
  int i, j;
    cmd_test(arg, num);

    if (alCore.alternative_num == ALT_NUM) {
	fprintf(stderr, "οĿ(%d)ĶƤޤ\n",
		ALT_NUM);
	exit(1);
    }

    strcpy(alCore.alternative[alCore.alternative_num].label, arg[1]);
    for (i = 0, j = 0; j < strlen(arg[2]);) {
	if (arg[2][j] < 0x80) {
	   alCore.alternative[alCore.alternative_num].string[i++] = arg[2][j];
	    j++;
	} else {
	  alCore.alternative[alCore.alternative_num].string[i++] = arg[2][j] << 8 | arg[2][j + 1];
	    j += 2;
	}
    }
    alCore.alternative[alCore.alternative_num].string[i] = 0;
    alCore.alternative_num++;

    return 1;
}

int
cmd_Sound(char arg[][ARG_LEN], int num)
{
    cmd_test(arg, num);

    return 1;
}

int
cmd_Staffroll(char arg[][ARG_LEN], int num)
{
    cmd_test(arg, num);
    return 1;
}

int
cmd_StopMusic(char arg[][ARG_LEN], int num)
{
    cmd_test(arg, num);
    return 1;
}

int
cmd_Time(char arg[][ARG_LEN], int num)
{
    cmd_test(arg, num);
    return 1;
}

typedef struct {
    char name[ARG_LEN];
    int (*func) (char[][ARG_LEN], int);
} Command;

Command command_list[] = {
    {"Bg", cmd_Bg},
    {"CGLook", cmd_CGLook},
    {"Chara", cmd_Chara},
    {"Effect", cmd_Effect},
    {"End", cmd_End},
    {"FadeOutMusic", cmd_FadeOutMusic},
    {"Flag", cmd_Flag},
    {"IfJump", cmd_IfJump},
    {"Jump", cmd_Jump},
    {"Light", cmd_Light},
    {"MessageType", cmd_MessageType},
    {"Music", cmd_Music},
    {"OutAllChara", cmd_OutAllChara},
    {"OutBg", cmd_OutBg},
    {"OutChara", cmd_OutChara},
    {"PREVLook", cmd_PREVLook},
    {"Page", cmd_Page},
    {"Return", cmd_Return},
    {"SceneName", cmd_SceneName},
    {"SelStart", cmd_SelStart},
    {"Select", cmd_Select},
    {"Sound", cmd_Sound},
    {"Staffroll", cmd_Staffroll},
    {"StopMusic", cmd_StopMusic},
    {"Time", cmd_Time},
};

int
proc_command()
{
    int i, num;
    char argv[ARG_NUM][ARG_LEN];

    num = parse_arg(command_buf + 1, argv);

    for (i = 0; i < sizeof(command_list) / sizeof(command_list[0]); i++) {
	if (!strcmp(command_list[i].name, argv[0])) {
	    return command_list[i].func(argv, num);
	}
    }
    fprintf(stderr, "̤ΤΥޥ[%s]Ǥ\n", argv[0]);
    exit(1);
}
