/* 
 * Tenshi no Hashigo for X
 * Copyright (c) 2001 TF <tf@denpa.org>
 * All rights reserverd.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *      This product includes software developed by TF.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: $
 */

#include <sys/types.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <string.h>

#include "aladder.h"

#define FILENAME_MAX 1024

void
setup_directory()
{
  char tmppath[FILENAME_MAX];

  struct stat sb;
  snprintf(tmppath, sizeof tmppath, "%s/%s", getenv("HOME"), ".angelladder");
  if (stat(tmppath, &sb) < 0) {
    perror(tmppath);
    fprintf(stderr, "%s: making...\n", tmppath);
    if (mkdir(tmppath, 0700)  < 0) {
      perror(tmppath);
      exit(1);
    }
  } else if (!(sb.st_mode & S_IFDIR)) {
    fprintf(stderr, "%s: not directory!\n", tmppath);
    exit(1);
  }

  alCore.dir.save_dir = strdup(tmppath);

  if (!(alCore.dir.app_dir = getenv("ANGELLADDER_DIR")))
    alCore.dir.app_dir = ANGELLADDER_DIR;
}

void
save_persistent_info()
{
    FILE *fp;
    FlagList *p;
    char path[FILENAME_MAX];

    snprintf(path, sizeof path, "%s/%s", alCore.dir.save_dir, "/state");
    if ((fp = fopen(path, "w")) == NULL) {
	perror("save_persistent_info");
	return;
    }

    fprintf(fp, "# ending\n");
    p = alCore.flag;
    while (p != NULL) {
	char *c;
	c = index(p->value, '_');
	if (c != NULL) {
	    if (!strncmp(c, "_end", 4)) {
		fprintf(fp, "%s\n", p->value);
	    }
	}
	p = p->next;
    }

    fprintf(fp, "# seen cg\n");
    p = alCore.cg_flag;
    while (p != NULL) {
	fprintf(fp, "%s\n", p->value);
	p = p->next;
    }

    fclose(fp);
}

void
load_persistent_info()
{
    FILE *fp;
    char buf[BUFSIZ];
    char flag[ARG_LEN];
    int mode = 0;
    char path[FILENAME_MAX];

    snprintf(path, sizeof path, "%s/%s", alCore.dir.save_dir, "/state");

#define INPUT_FLAG 1
#define INPUT_CG 2

    if ((fp = fopen(path, "r")) == NULL) {
	perror("load_persistent_info");
	return;
    }

    while (fgets(buf, BUFSIZ, fp) != NULL) {
	if (buf[0] == '#') {
	    if (!strncmp(buf, "# ending", 8)) {
		mode = INPUT_FLAG;
	    } else if (!strncmp(buf, "# seen cg", 8)) {
		mode = INPUT_CG;
	    } else {
		mode = 0;
	    }
	} else {
	    switch (mode) {
	    case INPUT_FLAG:
		sscanf(buf, "%s", flag);
		// fprintf(stderr, "%s\n", flag);
		add_flag(flag, &alCore.flag);
		break;
	    case INPUT_CG:
		sscanf(buf, "%s", flag);
		// fprintf(stderr, "%s\n", flag);
		add_flag(flag, &alCore.cg_flag);
		break;
	    default:
		fprintf(stderr, "ǥ󥰡ɾǤ.");
		exit(1);
	    }
	}
    }
}

void
save_game()
{
    save_persistent_info();
}

void
load_game()
{

}
