/* 
 * Tenshi no Hashigo for X
 * Copyright (c) 2001 TF <tf@denpa.org>
 * All rights reserverd.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *      This product includes software developed by TF.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: $
 */
#include "aladder.h"
#include "command.h"

#include <string.h>
#include <stdlib.h>
#include <sys/time.h>

#define SCR_CMD 0
#define SCR_TXT 1
#define SCR_EOF 2

typedef struct {
    struct timeval start;
    int msec;
} timer;

static void
init_timer(timer *tp)
{
    gettimeofday(&tp->start, 0);
    tp->msec = 0;
}

static void
get_timer(timer *tp)
{
    struct timeval tv;
    long s, us;

    gettimeofday(&tv, 0);
    s = tv.tv_sec - tp->start.tv_sec;
    us = tv.tv_usec - tp->start.tv_usec;

    tp->msec = s * 1000 + us / 1000;
}

static void
scan_script(void)
{
    u_char buf[SCR_BUF_LEN];
    char argv[ARG_NUM][ARG_LEN];
    SceneList *scene;

    alCore.script.cur_pos = 0;

    while (1) {
	switch (get_next_script(buf, False)) {
	case SCR_CMD:
	    parse_arg(buf + 1, argv);
	    if (!strcmp(argv[0], "SceneName")) {
		scene = (SceneList *) malloc(sizeof(SceneList));
		scene->next = alCore.script.scene_list;
		strcpy(scene->name, argv[1]);
		scene->offset = alCore.script.cur_pos;

		alCore.script.scene_list = scene;
	    }
	    break;
	case SCR_TXT:
	    break;
	case SCR_EOF:
	    goto end;
	}
    }
  end:
}

void
load_script(const char *name)
{
    free_script();
    alCore.script.data = get_text(alCore.script_arc, name);
    alCore.script.end_pos = strlen(alCore.script.data);

    scan_script();
    alCore.script.cur_pos = 0;
}

void
free_script(void)
{
    if (alCore.script.data != NULL) {
	free(alCore.script.data);
	alCore.script.data = NULL;
    }

    if (alCore.script.scene_list != NULL) {
	SceneList *p, *q;
	p = alCore.script.scene_list;
	while (p != NULL) {
	    q = p->next;
	    free(p);
	    p = q;
	}
	alCore.script.scene_list = NULL;
    }
}

int
get_next_script(u_char *buf, int running)
{
    int i = 0;
    u_char ch;

    if (alCore.script.cur_pos == alCore.script.end_pos) {
	return SCR_EOF;
    }

    if (alCore.script.data[alCore.script.cur_pos] == '#') {
	while (1) {
	    ch = buf[i++] = alCore.script.data[alCore.script.cur_pos++];
	    if (alCore.script.cur_pos == alCore.script.end_pos) {
		break;
	    }
	    if (ch == 0xa) {
		break;
	    }
	    if (ch == ')') {
		if (alCore.script.data[alCore.script.cur_pos] == 0xa)
		    alCore.script.cur_pos++;
		break;
	    }
	}
	buf[i] = '\0';

	if (running) {
	    set_command(buf);
	    push_stack(STAT_COMMAND);
	}
	return SCR_CMD;
    }

    while (1) {
	ch = buf[i++] = alCore.script.data[alCore.script.cur_pos++];
	if (alCore.script.cur_pos == alCore.script.end_pos) {
	    break;
	}
	if (ch == 0xa) {
	    break;
	}
	if (ch == '#') {
	    alCore.script.cur_pos--;
	    buf[i - 1] = '\0';
	    break;
	}
    }
    buf[i] = '\0';

    if (running) {
	set_text(buf);
	push_stack(STAT_TEXT);
    }
    return SCR_TXT;
}

int
parse_arg(const char *cmd, char argv[ARG_NUM][ARG_LEN])
{
    int i = 0, j = 0, p = 0;

    while (1) {
	argv[i][j++] = cmd[p++];
	if (cmd[p] == '(' || cmd[p] == ',') {
	    argv[i][j] = '\0';
	    i++;
	    p++;
	    j = 0;
	}
	if (cmd[p] == ')' || cmd[p] == 0xa) {
	    argv[i][j] = '\0';
	    i++;
	    break;
	}
    }
    return i;
}

void
push_stack(int val)
{
    if (alCore.state_stack_depth == STAT_STACK_LEN - 1) {
	fprintf(stderr, "stack overrun\n");
	exit(1);
    }
    alCore.state_stack[++alCore.state_stack_depth] = val;
}

int
pop_stack()
{
    if (alCore.state_stack_depth == 0) {
	fprintf(stderr, "stack underrun\n");
	exit(1);
    }
    return alCore.state_stack[alCore.state_stack_depth--];
}

#define current_state (alCore.state_stack[alCore.state_stack_depth])
#define is_stack_empty ((alCore.state_stack_depth == 0) ? 1 : 0)

void
run_engine(void)
{
    u_char buf[SCR_BUF_LEN];
    int ret, idle;
    timer t;

    load_persistent_info();

    while (1) {
	init_timer(&t);
      head:
	if (is_stack_empty) {
	    if (get_next_script(buf, True) == SCR_EOF)
		goto end;
	} else {
	    switch (current_state) {
	    case STAT_COMMAND:
		if (proc_command()) {
		    pop_stack();
		}
		break;
	    case STAT_TEXT:
		if (proc_text()) {
		    pop_stack();
		    if (alCore.wait.per_sentence) {
			push_stack(STAT_WAIT);
		    } else {
			goto head;
		    }
		}
		break;
	    case STAT_ALTN:
		if (proc_alternative()) {
		    pop_stack();
		}
		break;
	    case STAT_WAIT:
		if (proc_wait()) {
		    pop_stack();
		}
		break;
	    }
	}
	Flip();

	get_timer(&t);
	idle = WAIT_TIME - t.msec * 1000;
	if (idle > 0)
	    usleep(idle);
    }
  end:
    cmd_End(NULL, 0);
}

int
proc_wait()
{
    char ch;

    if (!alCore.dev.tapped) {
	return 0;
    }

    alCore.dev.tapped = False;
    return 1;
}

int
proc_alternative()
{

    if (!alCore.dev.tapped) {
	return 0;
    }
    alCore.dev.tapped = False;

    if (!(alCore.text.highlight & ATTR_ALTN_MASK)) {
	return 0;
    }

    {
	int res;
	char arg[ARG_NUM][ARG_LEN];

	res = (ATTR_ALL_MASK ^ ATTR_ALTN_MASK) & alCore.text.highlight;
	// fprintf(stderr, "=>%x\n", res);

	newpage();
	alCore.alternative_num = 0;
	strcpy(arg[1], alCore.alternative[res].label);
	cmd_Jump(arg, 1);

    }

    return 1;
}
